
# q-expansion of newform 8048.2.a.c, downloaded from the LMFDB on 04 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8048
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-1], [-4], [3], [-5], [1], [0], [-8], [-9], [-6], [2], [2], [-10], [-5], [1], [-6], [-4], [5], [-13], [-6], [6], [-16], [-9], [-6], [10], [8], [4], [-8], [2], [-11], [-6], [-1], [0], [-10], [0], [12], [0], [-10], [6], [7], [2], [-12], [-18], [-22], [-5], [8], [-22], [-5], [16], [-13], [27], [28], [12], [26], [-13], [-13], [10], [21], [10], [7], [-20], [-15], [-28], [-22], [2], [27], [26], [14], [30], [-26], [-18], [-14], [-19], [17], [11], [8], [-6], [-3], [21], [-18], [26], [-22], [16], [30], [-10], [-5], [24], [-16], [18], [-11], [12], [30], [-38], [40], [6], [-1], [-27], [27], [-30], [30], [12], [25], [6], [26], [-6], [4], [-12], [0], [-36], [-23], [23], [-2], [8], [-26], [1], [17], [-40], [-40], [-51], [23], [-3], [2], [-4], [24], [28], [-23], [-8], [-48], [-1], [-14], [37], [-36], [26], [-32], [-17], [6], [26], [-16], [-26], [4], [31], [18], [24], [12], [-20], [-21], [-31], [33], [-36], [12], [-16], [-18], [4], [-48], [26], [-42], [16], [-12], [-28], [-59], [-36], [-39], [-46], [-35], [2], [6], [-21], [-18], [-62], [38], [-61], [50], [-48], [-31], [35], [-45], [8], [-10], [-44], [11], [-1], [13], [46], [-35], [-15], [-12], [-50], [19], [14], [39], [-25], [-6], [-34], [-55], [24], [8], [32], [65], [-54], [-20], [-15], [37], [53], [-7], [52], [50], [19], [-18], [-21], [-12], [23], [-58], [-29], [64], [-46], [-30], [0], [38], [15], [-21], [6], [-59], [2], [-48], [-16], [-33], [23], [-25], [9], [31], [40], [-20], [-5], [70], [26], [-62], [-15], [-36], [-12], [63], [45], [-25], [-20], [-30], [-21], [45], [10], [24], [-44], [70], [-34], [-6], [-19], [1], [-22], [30], [42], [8], [-7], [-52], [44], [-52], [-44], [25], [-57], [1], [-33], [30], [22], [-2], [-16], [58], [-81], [28], [-5], [60], [-16], [-44], [-65], [-26], [-10], [-55], [41], [-6], [66], [-42], [9], [-58], [-49], [-73], [-57], [57], [12], [-20], [-8], [-32], [-16], [-84], [0], [50], [-71], [-22], [24], [-76], [34], [28], [12], [54], [31], [-30], [88], [60], [34], [66], [-27], [19], [-14], [-75], [12], [-3], [46], [-64], [75], [-4], [-22], [-34], [-16], [-56], [26], [21], [26], [78], [8], [51], [-76], [-64], [2], [12], [70], [32], [17], [-22], [11], [-75], [92], [-6], [26], [-18], [-5], [-44], [-92], [50], [58], [-16], [54], [-18], [25], [6], [78], [-32], [-44], [9], [-5], [0], [-48], [64], [-25], [-26], [-65], [9], [17], [21], [-52], [-19], [18], [-45], [-10], [9], [90], [38], [-2], [-37], [47], [-28], [28], [46], [53], [-43], [-91], [59], [-80], [-36], [-61], [-35], [-48], [17], [-2], [-38], [-31], [-70], [15], [78], [69], [-42], [18], [1], [-74], [29], [-7], [38], [32], [-9], [-70], [29], [20], [-82], [48], [98], [14], [-57], [31], [2], [-42], [6], [9], [4], [-21], [42], [-38], [-88]]
